<?php

/**
 * @file
 * StoryChief module admin settings.
 */

/**
 * Return the StoryChief global settings form.
 */
function storychief_admin_settings($form, &$form_state) {
  global $base_url;
  $form['connect'] = array(
    '#type' => 'fieldset',
    '#title' => t('Connect'),
    '#description' => '<p>' . t('Step 1: Connect your website to StoryChief') . ' <a target="_blank" href="https://help.storychief.io/en/articles/735174-connect-your-drupal-website">View setup details</a></p>',
  );
  $form['connect']['storychief_key'] = array(
    '#type' => 'textfield',
    '#title' => 'Encryption key',
    '#required' => TRUE,
    '#default_value' => variable_get('storychief_key', ''),
    '#description' => t('Your encryption key is given when you add a Drupal destination on StoryChief'),
  );
  $form['connect']['website_url'] = array(
    '#type' => 'textfield',
    '#title' => 'Website url',
    '#disabled' => TRUE,
    '#value' => $base_url,
    '#description' => t('Copy past this url in your Drupal destination configuration on StoryChief'),
  );

  $node_types = node_type_get_types();
  $node_type_options = array();
  foreach ($node_types as $key => $node_type) {
    $node_type_options[$key] = $node_type->name;
  }

  $form['node_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Node settings'),
    '#description' => '<p>' . t('Step 2: Choose a node type where StoryChief must save stories to') . '</p><label for="edit-storychief-node-type">' . t('Node type') . ' </label>',
  );
  $form['node_settings']['type'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('container-inline')),
  );
  $form['node_settings']['type']['storychief_node_type'] = array(
    '#type' => 'select',
    '#empty_option' => t('- Choose node type -'),
    '#default_value' => variable_get('storychief_node_type', ''),
    '#options' => $node_type_options,
  );
  $form['node_settings']['type']['actions'] = array('#type' => 'actions');
  $form['node_settings']['type']['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );
  $storychief_node_type = variable_get('storychief_node_type', '');
  $form['fields'] = array(
    '#type' => 'fieldset',
    '#title' => t('Field mapping'),
  );

  if ($storychief_node_type != '') {
    $form['fields']['table'] = array(
      '#type' => 'container',
      '#prefix' => '<p>' . t("Last step! Let's map the StoryChief fields to your node:") . '</p><table class="sticky-enabled tableheader-processed sticky-table"><thead></thead><tr><th>' . t('Source: Story Chief story') . '</th><th width="50%">' . t('Target: @type node', array('@type' => $node_type_options[$storychief_node_type])) . '</th></tr>',
      '#suffix' => '</table>',
    );
    $form['fields']['table']['content'] = array(
      '#type' => 'container',
      '#prefix' => '<tr class="odd"><td><strong>Content & Excerpt <span class="form-required" title="This field is required.">*</span></strong><br /><div class="description"> ' . t('This field is required and must be of type <i>@field</i>.<br />The excerpt will be inserted in the summary and can be used for teaser views.', array('@field' => t('Long text and summary'))) . '</div></td><td>',
      'storychief_field_content' => array(
        '#type' => 'select',
        '#empty_option' => t('- Choose field -'),
        '#title' => t('Content'),
        '#title_display' => 'attribute',
        '#default_value' => variable_get('storychief_field_content', ''),
        '#disabled' => TRUE,
        '#required' => TRUE,
        '#options' => array(),
        '#description' => t('No <i>@field</i> field found on <i>@node</i> node',
          array(
            '@field' => t('Long text and summary'),
            '@node' => $node_type_options[$storychief_node_type],
          )
        ),
      ),
      '#suffix' => '</td><tr>',
    );
    $form['fields']['table']['image'] = array(
      '#type' => 'container',
      '#prefix' => '<tr class="even"><td><strong>Cover image</strong><br /><div class="description"> ' . t('The cover image for teaser views and/or header image.') . '</div></td><td>',
      'storychief_field_image' => array(
        '#type' => 'select',
        '#empty_option' => t('- Choose field -'),
        '#title' => t('Cover image'),
        '#title_display' => 'attribute',
        '#default_value' => variable_get('storychief_field_image', ''),
        '#disabled' => TRUE,
        '#options' => array(),
        '#description' => t('No <i>@field</i> field found on <i>@node</i> node',
          array(
            '@field' => t('Image'),
            '@node' => $node_type_options[$storychief_node_type],
          )
        ),
      ),
      '#suffix' => '</td><tr>',
    );

    $form['fields']['table']['tags'] = array(
      '#type' => 'container',
      '#prefix' => '<tr class="odd"><td><strong>Tags</strong><br /><div class="description"> ' . t('Micro-categories for your story used to show related stories.') . '</div></td><td>',
      'storychief_field_tags' => array(
        '#type' => 'select',
        '#empty_option' => t('- Choose field -'),
        '#title' => t('Tags'),
        '#title_display' => 'attribute',
        '#default_value' => variable_get('storychief_field_tags', ''),
        '#disabled' => TRUE,
        '#options' => array(),
        '#description' => t('No <i>@field</i> field with multiple values found on <i>@node</i> node',
          array(
            '@field' => t('Term reference'),
            '@node' => $node_type_options[$storychief_node_type],
          )
        ),
      ),
      '#suffix' => '</td><tr>',
    );

    $form['fields']['table']['category'] = array(
      '#type' => 'container',
      '#prefix' => '<tr class="even"><td><strong>Category</strong><br /><div class="description"> ' . t('The general topic the story can be classified in.<br />Readers can browse specific categories to see all stories in the category.') . '</div></td><td>',
      'storychief_field_category' => array(
        '#type' => 'select',
        '#empty_option' => t('- Choose field -'),
        '#title' => t('Category'),
        '#title_display' => 'attribute',
        '#default_value' => variable_get('storychief_field_category', ''),
        '#disabled' => TRUE,
        '#options' => array(),
        '#description' => t('No <i>@field</i> field found on <i>@node</i> node',
          array(
            '@field' => t('Term reference'),
            '@node' => $node_type_options[$storychief_node_type],
          )
        ),
      ),
      '#suffix' => '</td><tr>',
    );

    $sc_core_node_fields = storychief_get_core_node_fields($storychief_node_type);

    foreach ($sc_core_node_fields as $sc_core_node_field_name => $sc_core_node_field) {
      $form['fields']['table'][$sc_core_node_field_name]['storychief_field_' . $sc_core_node_field_name]['#options'] = $sc_core_node_field;
      unset($form['fields']['table'][$sc_core_node_field_name]['storychief_field_' . $sc_core_node_field_name]['#disabled']);
      unset($form['fields']['table'][$sc_core_node_field_name]['storychief_field_' . $sc_core_node_field_name]['#description']);
    }
    // Load custom fields.
    $sc_custom_fields = _storychief_load_custom_fields();
    $sc_custom_node_fields = storychief_get_custom_node_fields($storychief_node_type);
    if (count($sc_custom_fields)) {
      $form['custom_fields'] = array(
        '#type' => 'fieldset',
        '#title' => t('Custom Field mapping'),
      );
      $form['custom_fields']['table'] = array(
        '#type' => 'container',
        '#prefix' => '<p>' . t("You've got custom fields nice! Let's map the custom fields to your node:") . '</p><table class="sticky-enabled tableheader-processed sticky-table"><thead></thead><tr><th>' . t('Source: StoryChief story') . '</th><th width="50%">' . t('Target: @type node', array('@type' => $node_type_options[$storychief_node_type])) . '</th></tr>',
        '#suffix' => '</table>',
      );
      foreach ($sc_custom_fields as $index => $sc_custom_field) {
        $rowCssClass = 'odd';
        if ($index % 2 == 0) {
          $rowCssClass = 'even';
        }

        $form['custom_fields']['table'][$sc_custom_field['sc_field_name']] = array(
          '#type' => 'container',
          '#prefix' => '<tr class="' . $rowCssClass . '"><td><strong>' . $sc_custom_field['sc_field_name'] . '</strong><br /><div class="description"> ' . $sc_custom_field['sc_field_type'] . '</div></td><td>',
          'storychief_custom_field_' . $sc_custom_field['sc_field_name'] => array(
            '#type' => 'select',
            '#empty_option' => t('- Choose field -'),
            '#default_value' => $sc_custom_field['field'],
            '#options' => $sc_custom_node_fields,
          ),
          '#suffix' => '</td><tr>',
        );
      }
    }
    $form['multiple_node_types'] = array(
      '#type' => 'fieldset',
      '#title' => t('Multiple node types'),
    );
    $form['multiple_node_types']['table'] = array(
      '#type' => 'container',
      '#prefix' => '<p>Using multiple node types as newsitems? <a target="_blank" href="https://help.storychief.io/en/articles/3943403-drupal-multiple-node-types">View setup details</a></p>',
      '#suffix' => '',
    );
    if (count($sc_custom_fields)) {
      $sc_custom_fields_options = array();
      foreach ($sc_custom_fields as $sc_custom_field) {
        $sc_custom_fields_options[$sc_custom_field['sc_field_name']] = $sc_custom_field['sc_field_name'];
      }
      $form['multiple_node_types']['table']['storychief_node_type_custom_field'] = array(
        '#type' => 'select',
        '#empty_option' => t('- Choose custom field -'),
        '#default_value' => variable_get('storychief_node_type_custom_field', ''),
        '#options' => $sc_custom_fields_options,
      );
    }
  }
  else {
    $form['fields']['empty'] = array(
      '#type' => 'markup',
      '#markup' => '<p><i>You need to select a node type before you can start mapping your fields</i></p>',
    );
  }
  $form['#submit'][] = 'storychief_admin_settings_submit';
  return system_settings_form($form);
}

/**
 * Get all node fields we can attach to StoryChief custom fields.
 *
 * @param string $node_type
 *   An string containing the node type.
 *
 * @return array
 *   The array containing the node fields.
 */
function storychief_get_custom_node_fields($node_type) {
  $fields = field_info_instances('node', $node_type);
  $storychief_fields = array();
  foreach ($fields as $field_name => $field) {
    if ($field['deleted'] == '0') {
      $storychief_fields[$field_name] = $field_name . ' (' . $field['label'] . ')';
    }
  }
  return $storychief_fields;
}

/**
 * Get all node fields we can attach to StoryChief core fields.
 *
 * @param string $node_type
 *   An string containing the node type.
 *
 * @return array
 *   The array containing the node fields.
 */
function storychief_get_core_node_fields($node_type) {
  $fields = field_info_instances('node', $node_type);
  $storychief_fields = array();
  foreach ($fields as $field_name => $field) {
    $field_info = field_info_field($field_name);
    if ($field['deleted'] == '0') {
      $storychief_field_value = $field_name . ' (' . $field['label'] . ')';
      switch ($field_info['type']) {
        case 'image':
          $storychief_fields['image'][$field_name] = $storychief_field_value;
          break;

        case 'text_with_summary':
          $storychief_fields['content'][$field_name] = $storychief_field_value;
          break;

        case 'taxonomy_term_reference':
          if ($field_info['cardinality'] == '-1' || (int) $field_info['cardinality'] > 1) {
            $storychief_fields['tags'][$field_name] = $storychief_field_value;
          }
          $storychief_fields['category'][$field_name] = $storychief_field_value;
          break;
      }
    }

  }
  return $storychief_fields;
}

/**
 * Custom submit handler for storychief_admin_settings form.
 *
 * Save custom field mapping.
 *
 * @param array $form
 *   An array containing the form data.
 * @param array $form_state
 *   An array containing the form state data.
 */
function storychief_admin_settings_submit(array $form, array &$form_state) {
  $sc_custom_fields = _storychief_load_custom_fields();
  foreach ($form_state['values'] as $field_name => $field_value) {
    foreach ($sc_custom_fields as $sc_custom_field) {
      if ($field_name == 'storychief_custom_field_' . $sc_custom_field['sc_field_name']) {
        $custom_field = array(
          'sc_field_name' => $sc_custom_field['sc_field_name'],
          'field' => $field_value,
        );
        _storychief_update_custom_field($custom_field);
      }
    }
  }
}
